from paymentsgate.transport import Response


class PaymentsgateError(Exception):
    pass


class APIError(PaymentsgateError):
    error: str
    message: str
    code: int
    data: object | None
    details: object | None

    def __init__(
        self,
        error: str,
        message: str,
        data: object | None,
        details: object | None,
        status: int,
    ) -> None:
        super().__init__(f"[{error}] {message} (status: {status})")
        self.error = error
        self.message = message
        self.data = data
        self.code = status
        self.details = details
        if details is not None:
            print("Error details:", self.details)
        if data is not None:
            print(self.data)
        # print(f"{self.error}: {self.message} code: {self.code} details: {self.details}")


class APIResponseError(APIError):
    def __init__(self, response: Response) -> None:
        super().__init__(
            response.json_body.get("error"),
            response.json_body.get("message"),
            response.json_body.get("data"),
            response.json_body.get("details"),
            response.status_code,
        )


class APIAuthenticationError(APIError):
    def __init__(self, response: Response) -> None:
        super().__init__(
            response.json_body.get("error"),
            response.json_body.get("message"),
            response.json_body.get("data"),
            response.json_body.get("details"),
            response.status_code,
        )



from pydantic import BaseModel


class Request(BaseModel):
    method: str
    path: str
    content_type: str = "application/json"
    headers: dict[str, str] | None = None
    body: dict | None = None
    noAuth: bool | None = False
    signature: bool | None = False


class Response(BaseModel):
    raw_body: bytes
    status_code: int
    json_body: dict | None = None

    @property
    def success(self) -> bool:
        return self.status_code < 400

    def cast(self, model: BaseModel, error: dict):
        if self.success:
            return model(**self.json_body)
        return error(
            self.json_body.get("error"),
            self.json_body.get("message"),
            self.json_body.get("data"),
            self.json_body.get("status"),
        )

    def __str__(self) -> str:
        return self.raw_body.decode("utf-8")



import enum
import base64
import hashlib
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives import hashes, serialization


class SignatureCheckMode(enum.StrEnum):
    none = enum.auto()
    decrypt_only = enum.auto()
    full = enum.auto()


def flatten_stringify(d: dict, count: int = 1) -> dict:
    def inner(d, count):
        res = {}
        for k, v in d.items():
            if type(v) is list:
                count += 1
            elif type(v) is dict:
                new_d, count = inner(v, count)
                res = res | new_d
            else:
                res[(k.lower(), count)] = str(v).lower() if type(v) is bool else str(v)
                count += 1
        return res, count

    (d, _) = inner(d, count)
    return d


class SignatureHelper:
    def __init__(
        self,
        private_key_data: str,
        password: str = None,
        mode: SignatureCheckMode = SignatureCheckMode.full,
    ) -> None:
        decoded_key = base64.decodebytes(private_key_data.encode("utf-8"))
        self.private_key = serialization.load_pem_private_key(
            decoded_key, password=password
        )
        self.mode = mode

    def check(self, api_signature: str, json_value: dict) -> bool:
        try:
            self._check_impl(api_signature, json_value)
        except ValueError:
            return False
        except:
            raise
        return True

    def sign(self, json_value: dict) -> str:
        digest = self.encode(json_value)
        return base64.encodebytes(
            self.private_key.sign(
                digest,
                padding.PSS(
                    mgf=padding.MGF1(hashes.SHA256()),
                    salt_length=padding.PSS.MAX_LENGTH,
                ),
                hashes.SHA256(),
            )
        ).decode("utf-8")

    def encode(self, json_value: dict) -> bytes:
        d = flatten_stringify(json_value)
        val = "".join(d[k] for k in sorted(d.keys()))
        return hashlib.sha256(val.encode("utf-8")).hexdigest().encode("utf-8")

    def check_raise(self, api_signatre: str, json_value: dict) -> None:
        self._check_impl(api_signatre, json_value)

    def _check_impl(self, api_signature: str, json_value: dict) -> None:
        if self.mode == SignatureCheckMode.none:
            return
        if len(api_signature) < 256:
            raise ValueError(
                "wrong api_signature length, check Webhook version.in project settings, should be 3!"
            )
        decoded_signature = base64.decodebytes(api_signature.encode("utf-8"))
        alg = hashes.SHA256()
        digest = self.private_key.decrypt(
            decoded_signature,
            padding.OAEP(mgf=padding.MGF1(algorithm=alg), algorithm=alg, label=None),
        )
        if self.mode == SignatureCheckMode.decrypt_only:
            return
        if self.encode(json_value) != digest:
            raise ValueError("Error while checking signature")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['paymentsgate']

package_data = \
{'': ['*']}

install_requires = \
['cryptography>=44.0.2,<45.0.0',
 'httpx>=0.28.1,<0.29.0',
 'jwt>=1.3.1,<2.0.0',
 'pydantic>=2.8.2,<3.0.0',
 'ruff>=0.11.7,<0.12.0',
 'tomli>=2.0.1,<3.0.0']

setup_kwargs = {
    'name': 'paymentsgate',
    'version': '1.5.6',
    'description': "PaymentsGate's Python SDK for REST API",
    'long_description': '\n# Paymentsgate Python SDK for Payments REST API\n\n\n## Requirements\n\n- Python >= 3.8.1\n- dependencies:\n  - [`requests`](https://github.com/kennethreitz/requests)\n  - [`pydantic`](https://docs.pydantic.dev/latest/)\n  - [`jwt`](https://pyjwt.readthedocs.io/en/stable/)\n  \n## Installation\n\nThe simplest way to install SDK is to use [PIP](https://docs.python.org/3/installing/):\n\n```bash\npip install paymentsgate\n```\n\n## Basic usage\n\n```python\nfrom paymentsgate import ApiClient, Credentials, Currencies\n\n\n# minimal configuration\nconfig = Credentials().fromFile(\'/path/to/credentials.json\');\n\n# create ApiClient\nclient = ApiClient(config, baseUrl=\'https://api.example.com\');\n\n# request quote\nres = cli.Quote(\n  {\n    "amount": 10.10,\n    "currency_from": Currencies.EUR,\n    "currency_to": Currencies.AZN,\n  }\n)\nprint(res);\n```\n\nThe `credentials.json` file is used to connect to the client and contains all necessary data to use the API. This file can be obtained in your personal cabinet, in the service accounts section. Follow the instructions in the documentation to issue new keys. If you already have keys, but you don\'t feel comfortable storing them in a file, you can use client initialization via variables. In this case, the key data can be stored in external storage instead of on the file system:\n\n```python\nfrom paymentsgate import ApiClient, Credentials\n\nconfig = Credentials(\n  account_id="00000000-4000-4000-0000-00000000000a" \n  public_key="LS0tLS1CRUdJTiBSU0EgUFJJVkFUNSUlFb3dJQk..."\n)\n\nclient = ApiClient(config, baseUrl=\'https://api.example.com\');\n\n...\n```\n*It is important to note that the data format for key transfer is base46.\n\n## Examples\n\n### create PayIn\n\n```python\nres = cli.PayIn(\n  {\n    "amount": 10.10,\n    "currency": Currencies.AZN,\n    "invoiceId": "INVOICE-112123124",\n    "clientId": "",\n    "successUrl": "https://example.com/success",\n    "failUrl": "https://example.com/fail",\n    "type": InvoiceTypes.m10\n  }\n)\nprint(res);\n```\n\n### create PayOut\n\n```python\nres = cli.PayOut(\n  {\n    "amount": 5.12,\n    "currencyTo": Currencies.EUR,\n    "invoiceId": "INVOICE-112123124",\n    "clientId": "CLIENT-003010023004",\n    "baseCurrency": CurrencyTypes.fiat,\n    "feesStrategy": FeesStrategy.add,\n    "recipient": {\n      "account_number": "4000000000000012",\n      "account_owner": "CARD HOLDER",\n      "type": CredentialsTypes.card\n    }\n  }\n)\nprint(res);\n```\n\n### Error handling\n\n```python\ntry:\n  res = cli.PayOut(\n    {\n      "amount": 5.12,\n      "currencyTo": Currencies.EUR,\n      "invoiceId": "INVOICE-112123124",\n      "clientId": "CLIENT-003010023004",\n      "baseCurrency": CurrencyTypes.fiat,\n      "feesStrategy": FeesStrategy.add,\n      "recipient": {\n        "account_number": "4000000000000012",\n        "account_owner": "CARD HOLDER",\n        "type": CredentialsTypes.card\n      }\n    }\n  )\n  print(res);\nexcept APIAuthenticationError as err:\n  print(f"Authentication fail: {err.message}")\nexcept APIResponseError as err:\n  print(f"Exception: {err.error}; Message: {err.message}")\n```',
    'author': 'PaymentsGate',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/paymentsgate/python-secure-api',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from enum import StrEnum


class AuthenticationRealms(StrEnum):
    production = "production"
    sandbox = "sandbox"


class ApiPaths(StrEnum):
    token_issue = "/auth/token"
    token_refresh = "/auth/token/refresh"
    token_revoke = "/auth/token/revoke"
    token_validate = "/auth/token/validate"
    invoices_payin = "/deals/payin"
    invoices_payout = "/deals/payout"
    invoices_payout_tlv = "/deals/tlv"
    invoices_info = "/deals/:id"
    invoices_credentials = "/deals/:id/credentials"
    assets_list = "/wallet"
    assets_deposit = "/wallet/deposit"
    banks_list = "/banks/find"
    appel_create = "/support/create"
    appel_list = "/support/list"
    appel_stat = "/support/statistic"
    fx_quote = "/fx/calculatenew"
    fx_quote_tlv = "/fx/tlv"


class Currencies(StrEnum):
    USDT = "USDT"
    RUB = "RUB"
    EUR = "EUR"
    USD = "USD"
    TRY = "TRY"
    BYN = "BYN"
    CNY = "CNY"
    JPY = "JPY"
    GEL = "GEL"
    AZN = "AZN"
    INR = "INR"
    AED = "AED"
    KZT = "KZT"
    UZS = "UZS"
    TJS = "TJS"
    EGP = "EGP"
    PKR = "PKR"
    IDR = "IDR"
    BDT = "BDT"
    GBP = "GBP"
    THB = "THB"
    KGS = "KGS"
    PHP = "PHP"
    ZAR = "ZAR"
    ARS = "ARS"
    GHS = "GHS"
    KES = "KES"
    NGN = "NGN"
    AMD = "AMD"
    XOF = "XOF"
    CAD = "CAD"
    AFN = "AFN"
    ALL = "ALL"
    AUD = "AUD"
    BAM = "BAM"
    BGN = "BGN"
    BHD = "BHD"
    BIF = "BIF"
    BND = "BND"
    BOB = "BOB"
    BRL = "BRL"
    BWP = "BWP"
    BZD = "BZD"
    CDF = "CDF"
    CHF = "CHF"
    CLP = "CLP"
    COP = "COP"
    CRC = "CRC"
    CVE = "CVE"
    CZK = "CZK"
    DJF = "DJF"
    DKK = "DKK"
    DOP = "DOP"
    DZD = "DZD"
    EEK = "EEK"
    ERN = "ERN"
    ETB = "ETB"
    GNF = "GNF"
    GTQ = "GTQ"
    HKD = "HKD"
    HNL = "HNL"
    HRK = "HRK"
    HUF = "HUF"
    ILS = "ILS"
    IQD = "IQD"
    IRR = "IRR"
    ISK = "ISK"
    JMD = "JMD"
    JOD = "JOD"
    KHR = "KHR"
    KMF = "KMF"
    KRW = "KRW"
    KWD = "KWD"
    LBP = "LBP"
    LKR = "LKR"
    LTL = "LTL"
    LVL = "LVL"
    LYD = "LYD"
    MAD = "MAD"
    MDL = "MDL"
    MGA = "MGA"
    MKD = "MKD"
    MMK = "MMK"
    MOP = "MOP"
    MUR = "MUR"
    MXN = "MXN"
    MYR = "MYR"
    MZN = "MZN"
    NAD = "NAD"
    NIO = "NIO"
    NOK = "NOK"
    NPR = "NPR"
    NZD = "NZD"
    OMR = "OMR"
    PAB = "PAB"
    PEN = "PEN"
    PLN = "PLN"
    PYG = "PYG"
    QAR = "QAR"
    RON = "RON"
    RSD = "RSD"
    RWF = "RWF"
    SAR = "SAR"
    SDG = "SDG"
    SEK = "SEK"
    SGD = "SGD"
    SOS = "SOS"
    SYP = "SYP"
    TND = "TND"
    TOP = "TOP"
    TTD = "TTD"
    TWD = "TWD"
    TZS = "TZS"
    UAH = "UAH"
    UGX = "UGX"
    UYU = "UYU"
    VEF = "VEF"
    VND = "VND"
    YER = "YER"
    ZMK = "ZMK"
    MNT = "MNT"

class Languages(StrEnum):
    EN = ("EN",)
    AZ = ("AZ",)
    UZ = ("UZ",)
    GE = ("GE",)
    TR = ("TR",)
    AE = ("AE",)
    RU = ("RU",)
    IN = ("IN",)
    AR = ("AR",)
    KG = "KG"


class Statuses(StrEnum):
    queued = "queued"
    new = "new"
    pending = "pending"
    paid = "paid"
    completed = "completed"
    disputed = "disputed"
    canceled = "canceled"
    expired = "expired"


class CurrencyTypes(StrEnum):
    fiat = "FIAT"
    crypto = "CRYPTO"


class InvoiceTypes(StrEnum):
  p2p = ("p2p",)
  c2c = ("c2c",)
  m10 = ("m10",)
  mpay = ("mpay",)
  sbp = ("sbp",)
  sbpqr = ("sbpqr",)
  iban = ("iban",)
  upi = ("upi",)
  imps = ("imps",)
  spei = ("spei",)
  pix = ("pix",)
  rps = ("rps",)
  ibps = ("ibps",)
  bizum = ("bizum",)
  rkgs = ("rkgs",)
  kgsphone = ("kgsphone",)
  krungthainext = ("krungthainext",)
  sber = ("sber",)
  kztphone = ("kztphone",)
  bkash = ("bkash",)
  nagad = ("nagad",)
  alipay = ("alipay",)
  accountegp = ("accountegp",)
  accountphp = ("accountphp",)
  sberqr = ("sberqr",)
  maya = ("maya",)
  gcash = ("gcash",)
  banktransferphp = ("banktransferphp",)
  banktransferars = ("banktransferars",)
  phonepe = ("phonepe",)
  freecharge = ("freecharge",)
  instapay = ("instapay",)
  vodafonecash = ("vodafonecash",)
  orangecash = ("orangecash",)
  razn = ("razn",)
  rtjs = ("rtjs",)
  skzt = ("skzt",)
  scny = ("scny",)
  vtbcny = ("vtbcny",)
  sgel = ("sgel",)
  seur = ("seur",)
  stry = ("stry",)
  sthb = ("sthb",)
  sberpay = ("sberpay",)
  tpay = ("tpay",)
  opay = ("opay",)
  moniepoint = ("moniepoint",)
  palmpay = ("palmpay",)
  wave = ("wave",)
  orangemoney = ("orangemoney",)
  moovmoney = ("moovmoney",)
  rtjscard = ("rtjscard",)
  ruzs = ("ruzs",)
  amobile = ("amobile",)
  payid = ("payid",)
  baridi = ("baridi",)
  multiwidget = ("multiwidget",)
  banktransfermad = ("banktransfermad",)
  cih = ("cih",)
  cashplus = ("cashplus",)
  elqr = ("elqr",)
  odengi = ("odengi",)
  banktransferdop = ("banktransferdop",)
  sinpemovil = ("sinpemovil",)
  tryqr = ("tryqr",)
  bsb = ("bsb",)
  banktransfermnt = ("banktransfermnt",)
  stcpay = ("stcpay",)
  upiqr = ("upiqr",)
  ifsc = ("ifsc",)
  tjsbank = ("tjsbank",)
  sortgbp = ("sortgbp",)
  unionpay = ("unionpay",)
  ecomazn = ("ecomazn",)
  ecomrub = ("ecomrub",)
  banktransferthb = ("banktransferthb",)
  banktransferzar = ("banktransferzar",)
  dcecomusd = ("dcecomusd",)
  dcecomeur = ("dcecomeur",)
  benefitpay = "benefitpay"

class EELQRBankALias(StrEnum):
    bakai = ("bakai",)
    mbank = ("mbank",)
    optima = ("optima",)
    kicb = ("kicb",)
    odengi = ("odengi",)
    demir = ("demir",)
    megapay = "megapay"


class CredentialsTypes(StrEnum):
    iban = ("iban",)
    phone = ("phone",)
    card = ("card",)
    fps = ("fps",)
    qr = ("qr",)
    account = ("account",)
    custom = "custom"


class RiskScoreLevels(StrEnum):
    unclassified = "unclassified"
    hr = "hr"  # highest risk
    ftd = "ftd"  # high risk
    trusted = "trusted"  # low risk


class CancellationReason(StrEnum):
    NO_MONEY = ("NO_MONEY",)
    CREDENTIALS_INVALID = ("CREDENTIALS_INVALID",)
    EXPIRED = ("EXPIRED",)
    PRECHARGE_GAP_UPPER_LIMIT = ("PRECHARGE_GAP_UPPER_LIMIT",)
    CROSS_BANK_TFF_LESS_THAN_3K = ("CROSS_BANK_TFF_LESS_THAN_3K",)
    CROSS_BANK_UNSUPPORTED = ("CROSS_BANK_UNSUPPORTED",)
    ACCOUNT_NUMBER_BLACKLISTED = ("ACCOUNT_NUMBER_BLACKLISTED",)
    ANTISPAM = ("ANTISPAM",)
    BY_CLIENT = ("BY_CLIENT",)
    BY_EXPIRATION = ("BY_EXPIRATION",)
    PAID_CANCEL_BY_EXPIRATION = "PAID_BY_EXPIRATION"


class FeesStrategy(StrEnum):
    add = "add"
    sub = "sub"


class InvoiceDirection(StrEnum):
    F2C = ("F2C",)
    C2F = ("C2F",)
    FIAT_IN = ("FIAT_IN",)
    FIAT_OUT = "FIAT_OUT"


class TTLUnits(StrEnum):
    sec = "sec"
    min = "min"
    hour = "hour"
